@@ -764,6 +764,10 @@ class IrrepsData:
     def shape(self):
         return self.contiguous.shape[:-1]
 
+    @property
+    def ndim(self):
+        return len(self.shape)
+
     def reshape(self, shape) -> "IrrepsData":
         list = [
             None if x is None else x.reshape(shape + (mul, ir.dim))
@@ -771,6 +775,15 @@ class IrrepsData:
         ]
         return IrrepsData(self.irreps, self.contiguous.reshape(shape + (self.irreps.dim,)), list)
 
+    def broadcast_to(self, shape) -> "IrrepsData":
+        assert isinstance(shape, tuple)
+        contiguous = jnp.broadcast_to(self.contiguous, shape + (self.irreps.dim,))
+        list = [
+            None if x is None else jnp.broadcast_to(x, shape + (mul, ir.dim))
+            for (mul, ir), x in zip(self.irreps, self.list)
+        ]
+        return IrrepsData(self.irreps, contiguous, list)
+
     def replace_none_with_zeros(self) -> "IrrepsData":
         list = [
             jnp.zeros(self.shape + (mul, ir.dim)) if x is None else x
@@ -878,7 +891,7 @@ class IrrepsData:
         k = (1 - d) / 2
         return self.transform_by_angles(*matrix_to_angles(R), k)
 
-    def convert(self, irreps):
+    def convert(self, irreps: Irreps) -> "IrrepsData":
         r"""Convert the list property into an equivalent irreps
 
         Args:
